==> web-10.0.0.tgz: package/lib/myTable/treeTable/TreeTable.js <==
import * as React from "react";
import { GroupCols } from "./GroupCols";
import TabView from "./TabView";
import { formatterTreeData, activeStatus, cascade } from "./formatterTreeData";
const compareFn = (a, b) => a - b;
class TreeTable extends React.PureComponent {
    constructor(props) {
        super(props);
        this.tabMainTabBodyDomArr = [];
        this.tabMainTabBodyScrollFnArr = [];
        this.changeState = (path, key) => {
            if (key === "expand") {
                this.expand(path);
            }
            else if (key === "active") {
                this.checkChild(path);
            }
            else if (key === "checkPar") {
                this.checkPar(path);
            }
        };
        this.changeScrollTop = (top, index) => {
            const fn = this.props.originScroll ? (val) => {
                val.scrollTop = top;
            } : (val, oindex) => {
                if (this.fieldArr[oindex].forzen) {
                    val.scrollTop = top;
                }
                else {
                    const methods = this.tabMainTabBodyScrollFnArr[oindex];
                    methods && methods.scrollToTop(top);
                }
            };
            this.tabMainTabBodyDomArr.map((val, oindex) => {
                if (oindex !== index) {
                    fn(val, oindex);
                }
            });
        };
        this.setTabBodyDom = (dom, index, scrollBoxFn) => {
            this.tabMainTabBodyDomArr[index] = dom;
            this.tabMainTabBodyScrollFnArr[index] = scrollBoxFn;
        };
        const { children, data, childField, idField, multiply, defaultSel, initSelectVal, } = this.props;
        this.fieldArr = this.getFieldArr(children);
        const obj = formatterTreeData({ childField: childField, idField, multiply }, defaultSel, data);
        this.fixObj = this.initFixObj();
        this.state = {
            immutabData: obj.data,
            preData: data,
            selectArr: obj.selectArr,
            preInitSelect: initSelectVal,
            isreLoad: false
        };
    }
    static getDerivedStateFromProps(nextProps, preState) {
        if (nextProps.data !== preState.preData ||
            nextProps.initSelectVal !== preState.preInitSelect) {
            const newData = nextProps.data;
            const { idField, childField, multiply } = nextProps;
            const defaultSel = nextProps.initSelectVal
                ? nextProps.initSelectVal.id
                : nextProps.defaultSel;
            const obj = formatterTreeData({
                idField,
                childField: childField,
                multiply,
            }, defaultSel, newData);
            return {
                preData: newData,
                immutabData: obj.data,
                preInitSelect: nextProps.initSelectVal,
                selectArr: obj.selectArr,
                isreLoad: !preState.isreLoad
            };
        }
        else {
            return null;
        }
    }
    getFieldArr(arr) {
        const { noOrder } = this.props;
        return React.Children.map(arr, function (val, index) {
            const { children: childArr, forzen, width: groupWidth } = val.props;
            let widTotal = 0;
            const child = React.Children.map(childArr, function (node) {
                const { children, width, field, formatter, align } = node.props;
                widTotal += width;
                return {
                    width,
                    field,
                    formatter,
                    text: children,
                    align,
                };
            });
            if (index === 0 && !noOrder) {
                const orderGroup = {
                    width: 60,
                    field: "order",
                    text: "序号",
                    formatter: undefined,
                    align: "center",
                };
                child.unshift(orderGroup);
                widTotal += 60;
            }
            return {
                child,
                width: forzen ? widTotal : groupWidth,
                forzen,
            };
        });
    }
    expand(path) {
        const { childField } = this.props;
        const arr = path.split(",").join(`,${childField},`)
            .split(",");
        const { immutabData } = this.state;
        let newNode = immutabData.getIn(arr);
        if (!newNode) {
            return;
        }
        this.setState((pre) => ({
            immutabData: pre.immutabData.updateIn(arr, (val) => {
                const expand = val.get("expand");
                return val.set("expand", !expand);
            }),
        }));
    }
    checkChild(path) {
        const { childField, idField, clickOrCheckForbid, tabField } = this.props;
        const { selectArr, immutabData } = this.state;
        const arr = path.split(",").join(`,${childField},`)
            .split(",");
        let _select = selectArr;
        let newNode = immutabData.getIn(arr);
        if (!newNode) {
            return;
        }
        if (clickOrCheckForbid && clickOrCheckForbid(newNode, tabField)) {
            return;
        }
        this.setState((pre) => {
            let data = pre.immutabData;
            data = data.updateIn(arr, (val) => {
                const active = val.get("active") === activeStatus.noSelect
                    ? activeStatus.select
                    : activeStatus.noSelect;
                if (active === activeStatus.select) {
                    const idVal = val.get(idField);
                    _select = _select.push(idVal);
                }
                else {
                    _select = _select.filter((_val) => _val !== val.get(idField));
                }
                return val.set("active", active);
            });
            data = cascade(path, data, childField);
            return {
                immutabData: data,
                selectArr: _select,
            };
        });
    }
    mapFn(list, active, select) {
        let _select = select;
        const { idField, childField } = this.props;
        const arr = list.map((val) => {
            let _child = val.get(childField);
            let _node = val;
            if (_child.size) {
                const result = this.mapFn(_child, active, _select);
                _node = _node.set(childField, result.arr);
                _select = result.selecte;
            }
            else {
                if (active === activeStatus.select) {
                    if (_node.get("active") === activeStatus.select) {
                        _select = _select.filter((_val) => _val !== _node.get(idField));
                    }
                    _select = _select.push(_node.get(idField));
                }
                else {
                    _select = _select.filter((_val) => _val !== _node.get(idField));
                }
            }
            return _node.set("active", active);
        });
        return {
            arr: arr,
            selecte: _select,
        };
    }
    checkPar(path) {
        const { childField, tabField, clickOrCheckForbid } = this.props;
        const { selectArr, immutabData } = this.state;
        const indexArr = path.split(",").join(`,${childField},`)
            .split(",");
        let _select = selectArr;
        let newNode = immutabData.getIn(indexArr);
        if (!newNode) {
            return;
        }
        if (clickOrCheckForbid && clickOrCheckForbid(newNode, tabField)) {
            return;
        }
        this.setState((pre) => {
            const data = pre.immutabData;
            let _data = data;
            _data = _data.updateIn(indexArr, (val) => {
                let node = val;
                const active = node.get("active") === activeStatus.select
                    ? activeStatus.noSelect
                    : activeStatus.select;
                node = node.withMutations((map) => {
                    let _child = map.get(childField);
                    let _map = map;
                    const result = this.mapFn(_child, active, selectArr);
                    _map = _map.set(childField, result.arr);
                    _select = result.selecte;
                    _map = _map.set("active", active);
                    return _map;
                });
                newNode = node;
                return node;
            });
            _data = cascade(path, _data, childField);
            return {
                immutabData: _data,
                selectArr: _select,
            };
        });
    }
    initFixObj() {
        const { tabField, idField, multiply, defaultSel, childField, emptyTxt, itemIcon, noOrder, } = this.props;
        return {
            tabField,
            emptyTxt,
            itemIcon,
            noOrder,
            idField,
            multiply,
            defaultSel,
            childField: childField,
        };
    }
    viewMap() {
        const { immutabData, isreLoad } = this.state;
        const { originScroll } = this.props;
        return this.fieldArr.map((group, index) => (React.createElement(TabView, { config: group, originScroll: originScroll, setTabBodyDom: this.setTabBodyDom, key: index, changeScrollTop: this.changeScrollTop, data: immutabData, viewIndex: index, changeState: this.changeState, fixObj: this.fixObj, isreLoad: isreLoad })));
    }
    setTabViewBottomFixHeight() {
        if (!this.props.originScroll) {
            return;
        }
        const arr = this.fieldArr;
        const res = this.tabMainTabBodyDomArr.findIndex((val, index) => {
            let status = false;
            if (!arr[index].forzen) {
                status = val.scrollWidth > val.clientWidth;
            }
            return status;
        });
        if (res !== -1) {
            this.tabMainTabBodyDomArr.forEach((val, index) => {
                if (arr[index].forzen) {
                    val.classList.add("tab-over-wid");
                }
                else if (res !== index) {
                    if (val.scrollWidth <= val.clientWidth) {
                        val.classList.add("tab-over-wid");
                    }
                }
            });
        }
        else {
            this.tabMainTabBodyDomArr.forEach((val) => {
                val.classList.remove("tab-over-wid");
            });
        }
    }
    componentDidMount() {
        this.setDom();
    }
    setSameH() {
        let domArr = this.tabMainTabBodyDomArr;
        if (domArr.length < 2) {
            return;
        }
        this.mapDom(domArr);
    }
    setDom() {
        this.whileDomH();
        this.setTabViewBottomFixHeight();
        if (!this.props.originScroll) {
            this.setState((pre) => ({
                isreLoad: !pre.isreLoad
            }));
        }
    }
    componentDidUpdate(preProps) {
        if (preProps.data !== this.props.data) {
            this.setDom();
        }
    }
    mapDom(domArr) {
        const tableArr = domArr.map((val) => val.firstElementChild);
        const tabHArr = tableArr.map((val) => val.clientHeight).sort(compareFn);
        if (tabHArr[0] !== tabHArr[tabHArr.length - 1]) {
            let viewTrArr = tableArr.map((val) => val.lastElementChild.children);
            [...viewTrArr[0]].map((val, index) => {
                const trHdom = viewTrArr.map((trArr) => trArr[index]);
                const trH = trHdom.map((vals) => vals.clientHeight).sort(compareFn);
                const trHMax = trH[trH.length - 1];
                if (trH[0] !== trHMax) {
                    if (val.classList.contains("tree-td")) {
                        const domArrs = trHdom.map((vals) => vals.firstChild.firstChild);
                        this.mapDom(domArrs);
                    }
                    else {
                        trHdom.forEach((tr) => {
                            [...tr.children].forEach((td) => {
                                td.style.height = `${trHMax}` + "px";
                            });
                        });
                    }
                }
            });
        }
    }
    whileDomH() {
        const rootRoot = this.tabMainTabBodyDomArr;
        let domArr = [rootRoot];
        while (domArr.length) {
            const contains = [];
            domArr.forEach((arr) => {
                const tabDomArr = arr.map((val) => val.firstElementChild);
                const tabHArr = tabDomArr
                    .map((val) => val.clientHeight)
                    .sort(compareFn);
                const maxH = tabHArr[tabHArr.length - 1];
                if (tabHArr[0] !== maxH) {
                    const trArr_Arr = tabDomArr.map((val) => val.lastElementChild.children);
                    [...trArr_Arr[0]].forEach((tr, index) => {
                        const trDomArr = trArr_Arr.map((trArr) => trArr[index]);
                        const trHArr = trDomArr
                            .map((val) => val.clientHeight)
                            .sort(compareFn);
                        const trHMax = trHArr[trHArr.length - 1];
                        if (tr.classList.contains("tree-td")) {
                            const childDomArr = trDomArr.map((val) => val.firstChild.firstChild.firstChild);
                            contains.push(childDomArr);
                        }
                        else {
                            trDomArr.forEach((val) => {
                                val.style.height = `${trHMax}px`;
                            });
                        }
                    });
                }
            });
            domArr = contains;
        }
    }
    render() {
        const { height } = this.props;
        const styleObj = height ? { height: height } : undefined;
        return (React.createElement("div", { className: "treeTap-wrap", style: styleObj },
            React.createElement("div", { className: "treeTab" }, this.viewMap())));
    }
}
TreeTable.defaultProps = {
    idField: "id",
    childField: "children",
    defaultSel: "",
};
export { TreeTable, GroupCols };

==> web-10.1.0.tgz: package/lib/myTable/treeTable/TreeTable.js <==
import * as React from "react";
import { GroupCols } from "./GroupCols";
import TabView from "./TabView";
import { formatterTreeData, activeStatus, cascade } from "./formatterTreeData";
const compareFn = (a, b) => a - b;
class TreeTable extends React.PureComponent {
    constructor(props) {
        super(props);
        this.tabMainTabBodyDomArr = [];
        this.tabMainTabBodyScrollFnArr = [];
        this.changeState = (path, key) => {
            if (key === "expand") {
                this.expand(path);
            }
            else if (key === "active") {
                this.checkChild(path);
            }
            else if (key === "checkPar") {
                this.checkPar(path);
            }
        };
        this.changeScrollTop = (top, index) => {
            const fn = this.props.originScroll ? (val) => {
                val.scrollTop = top;
            } : (val, oindex) => {
                if (this.fieldArr[oindex].forzen) {
                    val.scrollTop = top;
                }
                else {
                    const methods = this.tabMainTabBodyScrollFnArr[oindex];
                    methods && methods.scrollToTop(top);
                }
            };
            this.tabMainTabBodyDomArr.map((val, oindex) => {
                if (oindex !== index) {
                    fn(val, oindex);
                }
            });
        };
        this.setTabBodyDom = (dom, index, scrollBoxFn) => {
            this.tabMainTabBodyDomArr[index] = dom;
            this.tabMainTabBodyScrollFnArr[index] = scrollBoxFn;
        };
        const { children, data, childField, idField, multiply, defaultSel, initSelectVal, } = this.props;
        this.fieldArr = this.getFieldArr(children);
        const obj = formatterTreeData({ childField: childField, idField, multiply }, defaultSel, data);
        this.fixObj = this.initFixObj();
        this.state = {
            immutabData: obj.data,
            preData: data,
            selectArr: obj.selectArr,
            preInitSelect: initSelectVal,
            isreLoad: { reload: true }
        };
    }
    static getDerivedStateFromProps(nextProps, preState) {
        if (nextProps.data !== preState.preData ||
            nextProps.initSelectVal !== preState.preInitSelect) {
            const newData = nextProps.data;
            const { idField, childField, multiply } = nextProps;
            const defaultSel = nextProps.initSelectVal
                ? nextProps.initSelectVal.id
                : nextProps.defaultSel;
            const obj = formatterTreeData({
                idField,
                childField: childField,
                multiply,
            }, defaultSel, newData);
            return {
                preData: newData,
                immutabData: obj.data,
                preInitSelect: nextProps.initSelectVal,
                selectArr: obj.selectArr,
                isreLoad: preState.isreLoad
            };
        }
        else {
            return null;
        }
    }
    getFieldArr(arr) {
        const { noOrder } = this.props;
        return React.Children.map(arr, function (val, index) {
            const { children: childArr, forzen, width: groupWidth } = val.props;
            let widTotal = 0;
            const child = React.Children.map(childArr, function (node) {
                const { children, width, field, formatter, align } = node.props;
                widTotal += width;
                return {
                    width,
                    field,
                    formatter,
                    text: children,
                    align,
                };
            });
            if (index === 0 && !noOrder) {
                const orderGroup = {
                    width: 60,
                    field: "order",
                    text: "序号",
                    formatter: undefined,
                    align: "center",
                };
                child.unshift(orderGroup);
                widTotal += 60;
            }
            return {
                child,
                width: forzen ? widTotal : groupWidth,
                forzen,
            };
        });
    }
    expand(path) {
        const { childField } = this.props;
        const arr = path.split(",").join(`,${childField},`)
            .split(",");
        const { immutabData } = this.state;
        let newNode = immutabData.getIn(arr);
        if (!newNode) {
            return;
        }
        this.setState((pre) => ({
            immutabData: pre.immutabData.updateIn(arr, (val) => {
                const expand = val.get("expand");
                return val.set("expand", !expand);
            }),
        }));
    }
    checkChild(path) {
        const { childField, idField, clickOrCheckForbid, tabField } = this.props;
        const { selectArr, immutabData } = this.state;
        const arr = path.split(",").join(`,${childField},`)
            .split(",");
        let _select = selectArr;
        let newNode = immutabData.getIn(arr);
        if (!newNode) {
            return;
        }
        if (clickOrCheckForbid && clickOrCheckForbid(newNode, tabField)) {
            return;
        }
        this.setState((pre) => {
            let data = pre.immutabData;
            data = data.updateIn(arr, (val) => {
                const active = val.get("active") === activeStatus.noSelect
                    ? activeStatus.select
                    : activeStatus.noSelect;
                if (active === activeStatus.select) {
                    const idVal = val.get(idField);
                    _select = _select.push(idVal);
                }
                else {
                    _select = _select.filter((_val) => _val !== val.get(idField));
                }
                return val.set("active", active);
            });
            data = cascade(path, data, childField);
            return {
                immutabData: data,
                selectArr: _select,
            };
        });
    }
    mapFn(list, active, select) {
        let _select = select;
        const { idField, childField } = this.props;
        const arr = list.map((val) => {
            let _child = val.get(childField);
            let _node = val;
            if (_child.size) {
                const result = this.mapFn(_child, active, _select);
                _node = _node.set(childField, result.arr);
                _select = result.selecte;
            }
            else {
                if (active === activeStatus.select) {
                    if (_node.get("active") === activeStatus.select) {
                        _select = _select.filter((_val) => _val !== _node.get(idField));
                    }
                    _select = _select.push(_node.get(idField));
                }
                else {
                    _select = _select.filter((_val) => _val !== _node.get(idField));
                }
            }
            return _node.set("active", active);
        });
        return {
            arr: arr,
            selecte: _select,
        };
    }
    checkPar(path) {
        const { childField, tabField, clickOrCheckForbid } = this.props;
        const { selectArr, immutabData } = this.state;
        const indexArr = path.split(",").join(`,${childField},`)
            .split(",");
        let _select = selectArr;
        let newNode = immutabData.getIn(indexArr);
        if (!newNode) {
            return;
        }
        if (clickOrCheckForbid && clickOrCheckForbid(newNode, tabField)) {
            return;
        }
        this.setState((pre) => {
            const data = pre.immutabData;
            let _data = data;
            _data = _data.updateIn(indexArr, (val) => {
                let node = val;
                const active = node.get("active") === activeStatus.select
                    ? activeStatus.noSelect
                    : activeStatus.select;
                node = node.withMutations((map) => {
                    let _child = map.get(childField);
                    let _map = map;
                    const result = this.mapFn(_child, active, selectArr);
                    _map = _map.set(childField, result.arr);
                    _select = result.selecte;
                    _map = _map.set("active", active);
                    return _map;
                });
                newNode = node;
                return node;
            });
            _data = cascade(path, _data, childField);
            return {
                immutabData: _data,
                selectArr: _select,
            };
        });
    }
    initFixObj() {
        const { tabField, idField, multiply, defaultSel, childField, emptyTxt, itemIcon, noOrder, } = this.props;
        return {
            tabField,
            emptyTxt,
            itemIcon,
            noOrder,
            idField,
            multiply,
            defaultSel,
            childField: childField,
        };
    }
    viewMap() {
        const { immutabData, isreLoad } = this.state;
        const { originScroll } = this.props;
        const totalView = this.fieldArr.length;
        return this.fieldArr.map((group, index) => (React.createElement(TabView, { config: group, originScroll: originScroll, setTabBodyDom: this.setTabBodyDom, key: index, totalView: totalView, changeScrollTop: this.changeScrollTop, data: immutabData, viewIndex: index, changeState: this.changeState, fixObj: this.fixObj, isreLoad: isreLoad })));
    }
    setTabViewBottomFixHeight() {
        if (!this.props.originScroll) {
            return;
        }
        const arr = this.fieldArr;
        const res = this.tabMainTabBodyDomArr.findIndex((val, index) => {
            let status = false;
            if (!arr[index].forzen) {
                status = val.scrollWidth > val.clientWidth;
            }
            return status;
        });
        if (res !== -1) {
            this.tabMainTabBodyDomArr.forEach((val, index) => {
                if (arr[index].forzen) {
                    val.classList.add("tab-over-wid");
                }
                else if (res !== index) {
                    if (val.scrollWidth <= val.clientWidth) {
                        val.classList.add("tab-over-wid");
                    }
                }
            });
        }
        else {
            this.tabMainTabBodyDomArr.forEach((val) => {
                val.classList.remove("tab-over-wid");
            });
        }
    }
    componentDidMount() {
        this.setDom();
        const { noexpand, data } = this.props;
        if (noexpand && data.length) {
            const { childField, idField, multiply, defaultSel, } = this.props;
            const obj = formatterTreeData({ childField: childField, idField, multiply, noexpand }, defaultSel, data);
            window.setImmediate(() => {
                this.setState({
                    immutabData: obj.data,
                });
            });
        }
    }
    setSameH() {
        let domArr = this.tabMainTabBodyDomArr;
        if (domArr.length < 2) {
            return;
        }
        this.mapDom(domArr);
    }
    setDom() {
        this.whileDomH();
        this.setTabViewBottomFixHeight();
        if (!this.props.originScroll) {
            this.setState(() => ({
                isreLoad: { reload: true }
            }));
        }
    }
    componentDidUpdate(preProps) {
        if (preProps.data !== this.props.data) {
            this.setDom();
            const { noexpand, data } = this.props;
            if (noexpand && data.length) {
                const { childField, idField, multiply, defaultSel, } = this.props;
                const obj = formatterTreeData({ childField: childField, idField, multiply, noexpand }, defaultSel, data);
                window.setImmediate(() => {
                    this.setState({
                        immutabData: obj.data,
                    });
                });
            }
        }
    }
    mapDom(domArr) {
        const tableArr = domArr.map((val) => val.firstElementChild);
        const tabHArr = tableArr.map((val) => val.clientHeight).sort(compareFn);
        if (tabHArr[0] !== tabHArr[tabHArr.length - 1]) {
            let viewTrArr = tableArr.map((val) => val.lastElementChild.children);
            [...viewTrArr[0]].map((val, index) => {
                const trHdom = viewTrArr.map((trArr) => trArr[index]);
                const trH = trHdom.map((vals) => vals.clientHeight).sort(compareFn);
                const trHMax = trH[trH.length - 1];
                if (trH[0] !== trHMax) {
                    if (val.classList.contains("tree-td")) {
                        const domArrs = trHdom.map((vals) => vals.firstChild.firstChild);
                        this.mapDom(domArrs);
                    }
                    else {
                        trHdom.forEach((tr) => {
                            [...tr.children].forEach((td) => {
                                td.style.height = `${trHMax}` + "px";
                            });
                        });
                    }
                }
            });
        }
    }
    whileDomH() {
        const rootRoot = this.tabMainTabBodyDomArr;
        let domArr = [rootRoot];
        while (domArr.length) {
            const contains = [];
            domArr.forEach((arr) => {
                const tabDomArr = arr.map((val) => val.firstElementChild);
                const tabHArr = tabDomArr
                    .map((val) => val.clientHeight)
                    .sort(compareFn);
                const maxH = tabHArr[tabHArr.length - 1];
                if (tabHArr[0] !== maxH) {
                    const trArr_Arr = tabDomArr.map((val) => val.lastElementChild.children);
                    [...trArr_Arr[0]].forEach((tr, index) => {
                        const trDomArr = trArr_Arr.map((trArr) => trArr[index]);
                        const trHArr = trDomArr
                            .map((val) => val.clientHeight)
                            .sort(compareFn);
                        const trHMax = trHArr[trHArr.length - 1];
                        if (tr.classList.contains("tree-td")) {
                            const childDomArr = trDomArr.map((val) => val.firstChild.firstChild.firstChild);
                            contains.push(childDomArr);
                        }
                        else {
                            trDomArr.forEach((val) => {
                                val.style.height = `${trHMax}px`;
                            });
                        }
                    });
                }
            });
            domArr = contains;
        }
    }
    render() {
        const { height } = this.props;
        const styleObj = height ? { height: height } : undefined;
        return (React.createElement("div", { className: "treeTap-wrap", style: styleObj },
            React.createElement("div", { className: "treeTab" }, this.viewMap())));
    }
}
TreeTable.defaultProps = {
    idField: "id",
    childField: "children",
    defaultSel: "",
};
export { TreeTable, GroupCols };
